@@ -4,7 +4,7 @@
 # can do whatever you want with this stuff. If we meet some day, and you think
 # this stuff is worth it, you can buy me a beer in return.   Anton Berezin
 # ----------------------------------------------------------------------------
-# Copyright (c) 2005-2010 SPARTA, Inc.
+# Copyright (c) 2005-2011 SPARTA, Inc.
 # All rights reserved.
 #  
 # Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# $Id: Fast.pm 5202 2010-09-07 13:05:37Z hardaker $
+# $Id: Fast.pm 6169 2011-12-29 23:59:07Z hardaker $
 #
 package Net::DNS::ZoneFile::Fast;
 # documentation at the __END__ of the file
@@ -46,16 +46,23 @@ use Net::DNS;
 use Net::DNS::RR;
 use MIME::Base64;
 
-$VERSION = '1.14';
+$VERSION = '1.16';
 
 my $MAXIMUM_TTL = 0x7fffffff;
 
 my $pat_ttl = qr{\d+[\dwdhms]*}i;
 my $pat_skip = qr{\s*(?:;.*)?};
 my $pat_name = qr{[-\*\w\$\d\/*]+(?:\.[-\*\w\$\d\/]+)*};
-my $pat_maybefullname = qr{[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?};
 my $pat_maybefullnameorroot = qr{(?:\.|[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?)};
 
+#
+# Added the ability to have a backslash in the SOA username.  This is to
+# provide for the RFC-allowed "Joe\.Jones.example.com" construct to allow
+# dots in usernames.  Keeping the original version here for easy reference.
+#
+# my $pat_maybefullname = qr{[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?};
+my $pat_maybefullname   = qr{[-\w\$\d\/*\\]+(?:\.[-\w\$\d\/]+)*\.?};
+
 my $debug;
 my $domain;
 my $parse;
@@ -195,7 +202,11 @@ sub parse
 	    Net::DNS::RR->new_from_hash(%$z);
 
 	  if ($newrec->{'type'} eq 'DNSKEY') {
-	      $newrec->setkeytag;
+	      if (ref($newrec) ne 'Net::DNS::RR::DNSKEY') {
+		  warn "Failed to define a DNSSEC object (got: " . ref($newrec) . "); you're probably missing either MIME::Base64 or MIME::Base32";
+	      } else {
+		  $newrec->setkeytag;
+	      }
 	  }
 
 	  # no longer an issue with recent Net::DNS
@@ -281,7 +292,7 @@ sub parse_line
 	  				$offset ||= 0;
 	  				$width  ||= 0;
 	  				$base   ||= 'd';
-	  				sprintf "%$width$base", $offset + $from;
+	  				sprintf "%0$width$base", $offset + $from;
 	  			}xge;
 	  		$parse->();
 	  		$from++;
@@ -606,6 +617,33 @@ sub parse_line
 	  } else {
 	      error("bad txtdata in $type");
 	  }
+      } elsif (/\G(type[0-9]+)[ \t]+/igc) {
+          my $type = $1;
+	  if (/\G\\#\s+(\d+)\s+\(\s(.*)$/gc) {
+	      # multi-line
+	      $sshfp = { 
+			Line    => $ln,
+			name    => $domain,
+			type    => uc $type,
+			ttl     => $ttl,
+			class   => "IN",
+                        fptype  => $1,
+			fingerprint => $2,
+		       };
+	      $parse = \&parse_sshfp;
+	  } elsif (/\G\\#\s+(\d+)\s+(.*)$pat_skip$/gc) {
+	      push @zone, {
+			   Line    => $ln,
+			   name    => $domain,
+			   type    => uc $type,
+			   ttl     => $ttl,
+			   class   => "IN",
+                           fptype  => $1,
+			   fingerprint => $2,
+			  };
+	  } else {
+	      error("bad data in in $type");
+	  }
       } elsif (/\G(sshfp)[ \t]+/igc) {
 	  if (/\G(\d+)\s+(\d+)\s+\(\s*$/gc) {
 	      # multi-line
@@ -619,7 +657,7 @@ sub parse_line
 			fptype => $2,
 		       };
 	      $parse = \&parse_sshfp;
-	  } elsif (/\G(\d+)\s+(\d+)\s+(.*)$pat_skip$/gc) {
+	  } elsif (/\G(\d+)\s+(\d+)\s+([a-zA-Z0-9]+)$pat_skip$/gc) {
 	      push @zone, {
 			   Line    => $ln,
 			   name    => $domain,
@@ -842,6 +880,9 @@ sub parse_line
 	      $ds->{'digest'} .= $1;
 	      $ds->{'digest'} = lc($ds->{'digest'});
 	      $ds->{'digest'} =~ s/\s//g;
+	      # remove any surrounding single line ()s
+	      $ds->{'digest'} =~ s/^\(//;
+	      $ds->{'digest'} =~ s/\)$//;
 	      $ds->{'digestbin'} = pack("H*", $ds->{'digest'});
 	      push @zone, $ds;
 	      $ds = undef;
@@ -849,7 +890,7 @@ sub parse_line
 	      error("bad DS data");
 	  }
       } elsif (/\G(nsec)[ \t]+/igc) {
-	  if (/\G\s*($pat_maybefullnameorroot)\s+(.*)$pat_skip$/gc) {
+	  if (/\G\s*($pat_maybefullnameorroot)\s+(.*?)$pat_skip$/gc) {
 	      # XXX: set the typebm field ourselves?
 	      my ($nxtdname, $typelist) = ($1, $2);
 	      $typelist = join(" ",sort split(/\s+/,$typelist));
@@ -871,7 +912,7 @@ sub parse_line
       } elsif (/\G(nsec3)[ \t]+/igc) {
 	  error ("You are missing required modules for NSEC3 support")
 	    if (!$nsec3capable);
-	  if (/\G\s*(\d+)\s+(\d+)\s+(\d+)\s+([-0-9A-Fa-f]+)\s+($pat_maybefullname)\s+(.*)$pat_skip$/gc) {
+	  if (/\G\s*(\d+)\s+(\d+)\s+(\d+)\s+([-0-9A-Fa-f]+)\s+($pat_maybefullname)\s+(.*?)$pat_skip$/gc) {
 	      # XXX: set the typebm field ourselves?
 	      my ($alg, $flags, $iters, $salt, $nxthash, $typelist) =
 		($1, $2, $3, $4, $5, $6);
@@ -981,7 +1022,7 @@ sub parse_line
       } elsif (/\Gany\s+tsig.*$/igc) {
 	  # XXX ignore tsigs
       } else {
-	  error("unrecognized type");
+	  error("unrecognized type for $domain\n$_\n");
       }
   }
 
@@ -1466,7 +1507,7 @@ Copyright 2003 by Anton Berezin and catpipe Systems ApS
 
   Anton Berezin
 
-Copyright (c) 2004-2010 SPARTA, Inc.
+Copyright (c) 2004-2011 SPARTA, Inc.
   All rights reserved.
    
   Redistribution and use in source and binary forms, with or without
